//Calendar implementation for the HTWK Leipzig timetable. Evaluation and display of the individual dates in iCal format.
//Copyright (C) 2024 HTWKalender support@htwkalender.de

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU Affero General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU Affero General Public License for more details.

//You should have received a copy of the GNU Affero General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

package ical

import (
	"encoding/json"
	"github.com/pocketbase/dbx"
	"github.com/pocketbase/pocketbase"
	"htwkalender/model"
)

//update ical feed json
//add uuid field
//remove module name field

func MigrateFeedJson(app *pocketbase.PocketBase) error {

	records, err := app.Dao().FindRecordsByFilter("feeds", "1=1", "-created", 0, 0)
	if err != nil {
		return err
	}

	for _, feed := range records {

		var modules []model.FeedCollection

		err := json.Unmarshal([]byte(feed.GetString("modules")), &modules)
		if err != nil {
			return err
		}

		var uuidFeedCollections []model.FeedCollection

		for _, module := range modules {
			uuid := searchUUIDForModule(app, module)

			if uuid != "" {
				uuidFeedCollections = append(uuidFeedCollections, model.FeedCollection{UUID: uuid, Name: module.Name, Course: module.Course, UserDefinedName: module.UserDefinedName})
			}
		}

		jsonModules, _ := json.Marshal(uuidFeedCollections)
		feed.Set("modules", string(jsonModules))

		err = app.Dao().SaveRecord(feed)
		if err != nil {
			return err
		}
	}
	return nil
}

func searchUUIDForModule(app *pocketbase.PocketBase, module model.FeedCollection) string {
	var event model.Event
	err := app.Dao().DB().Select("*").From("events").Where(dbx.NewExp("Name = {:name} AND course = {:course}", dbx.Params{"name": module.Name, "course": module.Course})).One(&event)
	if err != nil {
		return ""
	}
	return event.UUID
}
